'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = inspectContent;

var _falafel = require('falafel');

var _falafel2 = _interopRequireDefault(_falafel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Method = function Method(type, name, parentClass, start, end, args, returns) {
  _classCallCheck(this, Method);

  var params = { type: type, name: name, parentClass: parentClass, start: start, end: end, args: args, returns: returns };
  Object.assign(this, params);
};

var ReturnResult = function ReturnResult(start, end) {
  var name = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];

  _classCallCheck(this, ReturnResult);

  var params = { start: start, end: end, name: name };
  Object.assign(this, params);
};

var Param = function Param(name) {
  var optional = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

  _classCallCheck(this, Param);

  this.name = name;
  this.optional = optional;
};

function _getReturns(node, returnsArr) {
  var start = node.start;
  var end = node.end;
  return returnsArr.filter(function (returnResult) {
    return Boolean(returnResult.start >= start && returnResult.end <= end);
  });
}

function _parseParams(params) {
  return params.map(function (param) {
    var result = void 0;
    switch (param.type) {
      case 'Identifier':
        result = new Param(param.name);
        break;
      case 'AssignmentPattern':
        result = new Param(param.left.name, true);
        break;
      default:
    }
    return result;
  });
}

var _typeHandlers = {
  FunctionDeclaration: function FunctionDeclaration(node, arr, returnsArr) {
    var name = node.id.name || false;
    var args = _parseParams(node.params);
    var returns = _getReturns(node, returnsArr);
    var result = new Method(node.type, name, false, node.start, node.end, args, returns);
    arr.push(result);
  },
  ArrowFunctionExpression: function ArrowFunctionExpression(node, arr, returnsArr) {
    if (node.parent.type === 'VariableDeclarator') {
      var name = node.parent.id.name;
      var args = _parseParams(node.params);
      var returns = _getReturns(node, returnsArr);
      var result = new Method(node.type, name, false, node.start, node.end, args, returns);
      arr.push(result);
    }
  },
  MethodDefinition: function MethodDefinition(node, arr, returnsArr) {
    var name = node.key.name;
    var args = _parseParams(node.value.params);
    var parentClass = false;
    try {
      parentClass = node.parent.parent.id.name;
    } catch (err) {/**/}
    var returns = _getReturns(node, returnsArr);
    var result = new Method(node.type, name, parentClass, node.start, node.end, args, returns);
    arr.push(result);
    console.log(node);
  },
  ReturnStatement: function ReturnStatement(node, arr, returnsArr) {
    var name = false;
    try {
      name = node.argument.name;
    } catch (err) {/**/}
    var result = new ReturnResult(node.start, node.end, name);
    returnsArr.push(result);
  }
};

function _getHandler(nodeType, handlerObj) {
  if ({}.hasOwnProperty.call(handlerObj, nodeType)) {
    return handlerObj[nodeType];
  }
  return false;
}

function inspectContent(content) {
  var _functions = [];
  var _returns = [];
  return new Promise(function (resolve) {
    (0, _falafel2.default)(content, {
      sourceType: 'module',
      ecmaVersion: '6'
    }, function (node) {
      var handler = _getHandler(node.type, _typeHandlers);
      if (handler) handler(node, _functions, _returns);
      if (node.end === content.length) resolve(_functions);
    });
  });
}
module.exports = exports['default'];